// DISPLAY5.CPP - display dispatcher

//                This is a prototype for object based graphics editor.

//                Demonstrates "classic" object oriented programming
//                style using lists of pointers to objects and
//                virtual function based dispatching.

// Jim Fawcett, CST 2-127, (315)443-4454


#include "slist.hpp"   // declarations for generic singly linked list

struct point {         // coordinates on graphic screen
  int x;
  int y;
  point(int newx, int newy) : x(newx), y(newy) { }
  point() : x(0), y(0) { }
  point& operator()(int xin, int yin) { x=xin; y=yin; return *this; }
  point& operator=(point &p)          { x=p.x; y=p.y; return *this; }
};

class DisplayObj {     // base class for graphics objects
		       // establishes facilities shared by
		       // and tailored for each of the objects
  private:
    point position;

  public:
    DisplayObj() { position.x = position.y = 0; }
    DisplayObj(point p) { position = p;}
    virtual ~DisplayObj() { }
    virtual point where() { return position; }
    virtual void  show()  { cout << "showing dobject\n"; }
    virtual void  hide()  { cout << "hiding  dobject\n"; }
    virtual void  addLabel() { };
};

class LabeledObj : public DisplayObj {   // objects with labels
  private:
    char *label;
  public:
    LabeledObj() { }
    LabeledObj(point p) : DisplayObj(p) { };
    ~LabeledObj() { }
    void addLabel(void) { cout << "labeling lobject\n"; }
    virtual void show() { cout << "showing lobject\n"; }
    virtual void hide() { cout << "hiding  lobject\n"; }
};

class unLabeledObj : public DisplayObj { // objects without labels
  public:
    unLabeledObj() { }
    unLabeledObj(point p) : DisplayObj(p) { };
    ~unLabeledObj() { };
    void addLabel(void) { cout << "no label for ulobject\n"; }
    virtual void show() { cout << "showing ulobject\n"; }
    virtual void hide() { cout << "hiding  ulobject\n"; }
};
/**/
class box : public LabeledObj {     // graphics symbol: box
  public:
    box() { }
    box(point p) : LabeledObj(p) { };
    void show() { cout << "showing box object\n"; }
    void hide() { cout <<  "hiding box object\n"; }
};

class circle : public LabeledObj {  // graphics symbol: circle
  public:
    circle() { }
    circle(point p) : LabeledObj(p) { };
    void show() { cout << "showing circle object\n"; }
    void hide() { cout <<  "hiding circle object\n"; }
};

class line : public LabeledObj {    // graphics symbol: line
  public:
    line() { }
    line(point p) : LabeledObj(p) { };
    void show() { cout << "showing line object\n"; }
    void hide() { cout <<  "hiding line object\n"; }
};

class title : public unLabeledObj { // graphics object: title
  public:
    title() { }
    title(point p) : unLabeledObj(p) { };
    void show() { cout << "showing title object\n"; }
    void hide() { cout <<  "hiding title object\n"; }
};

/*



			  DisplayObj
			       |
		   +-----------+------------+
		   |                        |
	       LabeledObj             unLabeledObj
		   |                        |
	     +-----+------+                 |
	     |     |      |                 |
	   line   box   circle            title

*/
void main() {  //---- display dispatcher ----------------------

  cout << "\nTesting Display List Dispatcher\n\n";

  slist<DisplayObj*> displayList;  // Create list of pointers
				   // for graphics objects.
  DisplayObj *do_ptr;

// collect specifications (type and location) of graphics objects
// from user and create the objects for display

  cerr << "Enter graphics symbol type (line, box, circle, title).\n"
       << "Enter graphics symbol coordinates (int x, int y):\n"
       << "To quit enter q for graphics symbol.\n";

// input with rudimentary tokenizing
  #define TOK_SIZE 20
  char token[TOK_SIZE+1];
  int x, y, i=0;
  point coord;
// loop collects information about graphics objects from user
  while (token != "q") {
    cin.width(TOK_SIZE);
    cin >> token;
    if (token[0] == 'q')
      break;
    cin >> x >> y;
    cout << "user enters: ";
    cout.width(10);
    cout << token << ": ";
    cout.width(4);
    cout << x << ", ";
    cout.width(4);
    cout << y << "\n";

  // parse command input
    do_ptr = 0;
    if (token[0] == 'l')                // line command
      do_ptr = new line(coord(x,y));
    else if (token[0] == 'b')           // box command
      do_ptr = new box(coord(x,y));
    else if (token[0] == 'c')           // circle command
      do_ptr = new circle(coord(x,y));
    else if (token[0] == 't')           // title command
      do_ptr = new title(coord(x,y));
    if (do_ptr)                         // add to display list
      displayList.insert(i++, do_ptr);
    else
      cerr << "don't recognize symbol: " << token
	   << " please re-enter\n";
  }

// Now label the graphics objects.
  cout << "\nadding labels to objects in display list\n";
  for (i=0; i<displayList.listLength(); i++)
    displayList[i]->addLabel();

// Display graphics objects.
  cout << "\nredrawing the display list objects\n";
  for (i=0; i<displayList.listLength(); i++)
    displayList[i]->show();
}
